function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/*
 * Choice JS
*/

class Builder {
  //Constructor
  constructor(target, userOptions) {
    /**
     * Move the element to the parent
     *
    */
    _defineProperty(this, "moveToParent", choice => {
      var parentPosition = choice.getBoundingClientRect();

      //Change position
      this.dropdown.style.top = "".concat(parentPosition.top + choice.offsetHeight + 6, "px");
      this.dropdown.style.left = "".concat(parentPosition.left, "px");
    });
    /**
     * Block observer
    */
    _defineProperty(this, "visibilityObserver", new IntersectionObserver(function (entries, self) {
      //For each of the entries
      entries.forEach(entry => {
        //If the image is intersecting
        if (!entry.isIntersecting) {
          //Get elem
          var choice = entry.target,
            choiceid = choice.getAttribute('id');
          //Hide
          document.querySelector(".choicejs-dropdown-wrapper[data-attached-choicejs-id=\"".concat(choiceid, "\"]")).classList.remove('active');
          //Unobserve
          self.unobserve(choice);
          //On scroll
          window.removeEventListener('scroll', () => this.moveToParent(choice));
        }
      });
    }, {}));
    this.target = target;
    //Build our default options
    this.options = _objectSpread(_objectSpread({}, this.buildDefaultOptions(userOptions)), userOptions);
  }

  /**
   * Create a new choice dropdown
   *
   * @return {Promise}
  */
  create() {
    var _this = this;
    return _asyncToGenerator(function* () {
      //Build it
      var choice = yield _this.build();
      _this.choice = choice;
      //Deploy the select (where to?)
      _this.target.parentNode.insertBefore(choice, _this.target.nextSibling);
      //Toast bindings
      yield _this.bindings(choice);
      //Dispatch event to load it
      _this.target.dispatchEvent(new Event('change'));
      //Done
      return choice;
    })();
  }

  /**
   * Build the default options
   *
   * @param {Object} userOptions The user options for any overwrites
   * @return {Object} An object of options
  */
  buildDefaultOptions(userOptions) {
    var options = {
      width: 'inherit',
      height: 'inherit',
      placeholder: this.target.hasAttribute('placeholder') ? this.target.getAttribute('placeholder') : false,
      class: '',
      closeOnSelect: false,
      event: 'change',
      copyAttributes: true,
      bindTo: 'parent'
    };
    //Return
    return options;
  }

  /**
   * Configure options for the choice
   *
   * @param {HTMLElement} choice - The select we're configuring options for
   * @return {Promise}
  */
  configureOptions(choice) {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      //Width and height
      choice.style.width = _this2.options.width == 'inherit' ? _this2.target.offsetWidth + 'px' : _this2.options.width, choice.style.minHeight = _this2.options.height == 'inherit' ? _this2.target.offsetHeight + 'px' : _this2.options.height;

      //Class
      if (_this2.options.class != '') choice.classList.add(_this2.options.class);

      //Placeholder
      if (_this2.options.placeholder) choice.querySelector('.choicejs-select').innerHTML = _this2.options.placeholder;

      //Return
      return;
    })();
  }

  /**
   * Add option
   *
   * @param {HTMLElement} option - The option we're selecting
   * @returns {Promise}
  */
  addOption(option) {
    var _this3 = this;
    return _asyncToGenerator(function* () {
      //Get elems
      var value = option.getAttribute('value'),
        multiple = _this3.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this3.target.getAttribute('multiple'));
      //Mark the option as selected
      _this3.target.querySelector("option[value=\"".concat(value, "\"]")).selected = true;
      //If its not multiple, set the value of the parent select box
      if (!multiple) {
        _this3.target.value = value;
      }
    })();
  }

  /**
   * Remove option
   *
   * @param {HTMLElement} option - The option we're selecting
   * @returns {Promise}
  */
  removeOption(option) {
    var _this4 = this;
    return _asyncToGenerator(function* () {
      //Get elems
      var value = option.getAttribute('value'),
        multiple = _this4.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this4.target.getAttribute('multiple'));
      //Remove the hidden option
      _this4.target.querySelector("option[value=\"".concat(value, "\"]")).selected = false;
      _this4.target.querySelector("option[value=\"".concat(value, "\"]")).removeAttribute('selected');
      //Single or multiple select?
      if (multiple) {
        //If there is still options selected, return now, don't reset the value and index of the select
        if (_this4.target.querySelectorAll("option:checked").length > 0) return;
      }
      //Reset
      _this4.target.value = '';
      _this4.target.selectedIndex = -1;
    })();
  }

  /* ------------------------------------------------------------------------------------------------------------------------------------------------
  |  HTML Builders
  ------------------------------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Build the choice
   *
   * @return {Promise} HTMLElement The new choice
  */
  build() {
    var _this5 = this;
    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var wrapper = document.createElement('div'),
        select = document.createElement('div'),
        choiceId = 'choice-' + Math.random().toString(36).substr(2, 4);

      //Set attributes
      wrapper.classList.add('choicejs');
      wrapper.setAttribute('id', choiceId);
      select.classList.add('choicejs-select');

      //Set the id on the target so we can relate
      _this5.target.setAttribute('data-choicejs-id', "#".concat(wrapper.id));

      //Copy the select boxes attributes
      if (_this5.options.copyAttributes === true) {
        //Only copy over limited attributes
        wrapper.setAttribute('name', _this5.target.getAttribute('name'));
      }

      //Build the whole select and return it
      wrapper.appendChild(select);

      //Configure the options for it
      yield _this5.configureOptions(wrapper);

      //Return
      return wrapper;
    })();
  }

  /**
   * Build dropdown
   *
   * @returns {Promise}
  */
  buildDropdown() {
    var _this6 = this;
    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var choice = _this6.choice,
        search = document.createElement('input'),
        dropdownWrapper = document.createElement('div'),
        dropdown = document.createElement('div'),
        list = document.createElement('ul'),
        choiceId = _this6.choice.getAttribute('id'),
        wrapper = document.getElementById(choiceId);

      //Set attributes
      search.classList.add('choicejs-search');
      search.setAttribute('placeholder', 'Search');
      dropdownWrapper.classList.add('choicejs-dropdown-wrapper');
      dropdownWrapper.setAttribute('data-attached-choicejs-id', choiceId), dropdown.classList.add('choicejs-dropdown');

      //Add option function
      var addOption = /*#__PURE__*/function () {
        var _ref = _asyncToGenerator(function* (option, wrapper) {
          var item = document.createElement('li');
          item.innerHTML = option.innerHTML;
          //Copy all attributes over from the option
          if (_this6.options.copyAttributes === true) {
            for (var i = 0; i < option.attributes.length; i++) {
              item.setAttribute(option.attributes[i].nodeName, option.attributes[i].nodeValue);
            }
          }
          wrapper.appendChild(item);
        });
        return function addOption(_x, _x2) {
          return _ref.apply(this, arguments);
        };
      }();

      //Are we working with optgroups?
      var optgroups = _this6.target.querySelectorAll('optgroup');
      if (optgroups.length > 0) {
        for (var optgroup of optgroups) {
          var group = document.createElement('div');
          group.classList.add('optgroup');
          group.setAttribute('label', optgroup.getAttribute('label'));
          //Add the options to the opt group
          for (var option of optgroup.querySelectorAll('option')) {
            if (option.value != '' && option.value != undefined) addOption(option, group);
          }
          //Add the group to the list
          list.appendChild(group);
        }
      } else {
        //Loop the targets options and build our list
        for (var _option of _this6.target.querySelectorAll('option')) {
          if (_option.value != '' && _option.value != undefined) addOption(_option, list);
        }
      }

      //Build the whole select and return it
      dropdown.appendChild(search);
      dropdown.appendChild(list);
      dropdownWrapper.appendChild(dropdown);

      //Where are we attaching the fixed popup?
      switch (_this6.options.bindTo) {
        case 'parent':
          wrapper.appendChild(dropdownWrapper);
          break;
        case 'body':
          document.querySelector('body').appendChild(dropdownWrapper);
          break;
      }

      //Add the dropdown to the class for easy access
      _this6.dropdown = dropdownWrapper;

      //If the user has provided a search url and the `data-choice-search-preload` attribute, run a keyup to pre-load some data
      if (_this6.target.hasAttribute('data-choice-search-url') && _this6.target.hasAttribute('data-choice-search-preload')) {
        _this6.dropdown.querySelector('.choicejs-search').dispatchEvent(new Event('keyup'));
      }

      //Bindings
      yield Promise.all([_this6.bindOptionSelect(choice), _this6.bindOptionFiltering(choice), _this6.bindDynamicSearch(choice), _this6.bindEventListener(choice)]);

      //Return
      return dropdownWrapper;
    })();
  }

  /**
   * Build option pill
   *
   * @param {Node} option - The option selected
   * @param {Node} select - The select we're appending to
   * @returns {Promise}
  */
  addOptionPill(option, select) {
    var _this7 = this;
    return _asyncToGenerator(function* () {
      var wrapper = document.createElement('div'),
        close = document.createElement('span'),
        value = option.getAttribute('value');
      //Attributes
      wrapper.classList.add('choicejs-pill');
      wrapper.setAttribute('value', value);
      wrapper.innerHTML = option.innerHTML;
      //Build it
      wrapper.appendChild(close);
      select.appendChild(wrapper);
      //Add the remove event listener
      close.addEventListener('click', /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (e) {
          //Run remove btn
          yield _this7.removeOption(option);
          //What parent event to trigger?
          if (_this7.options.event !== false) {
            _this7.target.dispatchEvent(new Event(_this7.options.event));
          }
        });
        return function (_x3) {
          return _ref2.apply(this, arguments);
        };
      }());
      //Return
      return;
    })();
  }

  /* ------------------------------------------------------------------------------------------------------------------------------------------------
  |  Bindings
  ------------------------------------------------------------------------------------------------------------------------------------------------ */

  /**
   * Bind Event Listener
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */
  bindEventListener(choice) {
    var _this8 = this;
    return _asyncToGenerator(function* () {
      //Dropdown
      var dropdown = _this8.dropdown;
      //When an event is fired on the target, change our dropdowns
      _this8.target.addEventListener(_this8.options.event, /*#__PURE__*/_asyncToGenerator(function* () {
        //Rebuild the options and pills on the target change
        var multiple = _this8.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this8.target.getAttribute('multiple')),
          choice_select = choice.querySelector('.choicejs-select');
        //Reset the select placeholder
        choice_select.innerHTML = _this8.options.placeholder ? _this8.options.placeholder : '';
        //Reset all of the options
        for (var option of dropdown.querySelectorAll("ul li")) {
          option.removeAttribute('selected');
        }
        //Are we working with multiple?
        if (!multiple) {
          var selected_value = _this8.target.value;
          //Does the select have a value?
          if (selected_value != '') {
            var choice_option = dropdown.querySelector("ul li[value=\"".concat(selected_value, "\"]"));
            //Select it
            choice_option.setAttribute('selected', true);
            choice_select.innerHTML = choice_option.innerHTML;
          }
        } else {
          var pills = [],
            selected_options = _this8.target.querySelectorAll('option:checked');
          if (selected_options.length > 0) {
            choice_select.innerHTML = '';
            for (var _option2 of selected_options) {
              var _choice_option = dropdown.querySelector("ul li[value=\"".concat(_option2.value, "\"]"));
              _choice_option.setAttribute('selected', true);
              //Add a pillbox
              pills.push(_this8.addOptionPill(_choice_option, choice_select));
            }
          }
          yield Promise.all(pills);
        }
      }));
    })();
  }
  /**
   * Bind active click
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */
  bindActiveListener(choice) {
    var _this9 = this;
    return _asyncToGenerator(function* () {
      //Get the visilibity observer here for scope
      var observer = _this9.visibilityObserver,
        moveToParent = _this9.moveToParent,
        dropdown;

      //Close listener function
      var closeListener = function closeListener(event) {
        if (!choice.contains(event.target) && !dropdown.contains(event.target)) {
          dropdown.remove();
          //Unobserve
          observer.unobserve(choice);
          //On scroll
          window.removeEventListener('scroll', () => moveToParent(choice));
          //Close listener
          document.removeEventListener('click', closeListener);
        }
      };

      //Listeners
      choice.addEventListener('click', /*#__PURE__*/_asyncToGenerator(function* () {
        //Create the dropdown
        dropdown = yield _this9.buildDropdown();
        //Figure out width
        var wrapperWidth = choice.offsetWidth;
        dropdown.querySelector('.choicejs-dropdown').style.width = "".concat(wrapperWidth, "px");
        _this9.moveToParent(choice);
        //Open
        dropdown.classList.add('active');
        //Observe for visibility
        observer.observe(choice);
        //On scroll
        window.addEventListener('scroll', () => moveToParent(choice), true);
        //Close listener
        document.addEventListener('click', closeListener);
      }));
      //Return
      return;
    })();
  }

  /**
   * Bind option select
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */
  bindOptionSelect(choice) {
    var _this10 = this;
    return _asyncToGenerator(function* () {
      var dropdown = _this10.dropdown;
      //Loop and add
      var _loop = function* _loop(option) {
        //Add the event listener
        option.addEventListener('click', /*#__PURE__*/function () {
          var _ref5 = _asyncToGenerator(function* (e) {
            //Stop Prop
            e.stopPropagation();
            e.stopImmediatePropagation();
            //Add or remove
            if (option.hasAttribute('selected')) {
              //Remove the option
              yield _this10.removeOption(option);
            } else {
              //Add the option
              yield _this10.addOption(option);
            }
            //Close on select?
            if (_this10.options.closeOnSelect) {
              dropdown.classList.remove('active');
            }
            //What parent event to trigger?
            if (_this10.options.event !== false) {
              _this10.target.dispatchEvent(new Event(_this10.options.event));
            }
          });
          return function (_x4) {
            return _ref5.apply(this, arguments);
          };
        }());
      };
      for (var option of dropdown.querySelectorAll('ul li')) {
        yield* _loop(option);
      }
      //Return
      return;
    })();
  }

  /**
   * Bind option filtering. When a user types into the search box
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */
  bindOptionFiltering(choice) {
    var _this11 = this;
    return _asyncToGenerator(function* () {
      //If we have a dynamic url, do not bind to the list
      if (_this11.target.hasAttribute('data-choice-search-url')) return;
      //When the choice search bar changes
      var searchTimeout = 0,
        searchBar = _this11.dropdown.querySelector('.choicejs-search'),
        options = _this11.dropdown.querySelectorAll('ul li');
      //When the postcode is changed, test it
      searchBar.addEventListener('input', () => {
        //Clear the above timeout
        clearTimeout(searchTimeout);
        //Set a new timeout
        searchTimeout = setTimeout(() => {
          //Loop all options
          for (var option of options) {
            if (option.innerHTML.toLowerCase().includes(searchBar.value.toLowerCase())) {
              option.classList.remove('hide');
            } else {
              option.classList.add('hide');
            }
          }
        }, 500);
      });
      //Return
      return;
    })();
  }

  /**
   * Bind dynamic search if the user has provided an endpoint to use
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */
  bindDynamicSearch(choice) {
    var _this12 = this;
    return _asyncToGenerator(function* () {
      //Has the user also asked for multiple? If yes, warn and quit
      var multiple = _this12.target.hasAttribute('multiple') && ['true', 'multiple'].includes(_this12.target.getAttribute('multiple'));
      if (multiple) {
        console.info("Choice JS - Multiple options is not supported with live searching.");
        return;
      }
      //Get the url endpoint if we have one
      if (_this12.target.hasAttribute('data-choice-search-url')) {
        //Get the endpoint we're using
        var endpoint = _this12.target.getAttribute('data-choice-search-url'),
          searchBar = _this12.dropdown.querySelector('.choicejs-search'),
          choiceList = _this12.dropdown.querySelector('ul');
        //Watch for the search change
        var searchTimeout = 0;
        //Remove the bindings to avoid duplicates
        searchBar.addEventListener('keyup', _listener => {
          //Try
          try {
            clearTimeout(searchTimeout);
            //Set timeout
            searchTimeout = setTimeout( /*#__PURE__*/_asyncToGenerator(function* () {
              //Clear the select options and the choice options
              choiceList.innerHTML = '';
              _this12.target.innerHTML = '';
              //Get the user input, allow empty so the user can check in backend if empty and return default set
              var userInput = searchBar.value;
              //Build our params
              var url = new URL(endpoint);
              url.search = new URLSearchParams({
                choicesearch: userInput
              }).toString();
              //Run the search
              var response = yield fetch(url),
                result = yield response.json();
              for (var option of result) {
                //Create a new item
                var choiceItem = document.createElement('li'),
                  selectItem = document.createElement('option');
                //Set the visible value
                choiceItem.innerHTML = option.label;
                selectItem.innerHTML = option.label;
                //Set the actual value
                choiceItem.setAttribute('value', option.value);
                selectItem.value = option.value;
                //Append it to choice
                choiceList.appendChild(choiceItem);
                //Append it to our select
                _this12.target.appendChild(selectItem);
              }
              //Re-run our choice bindings
              _this12.bindOptionSelect(choice);
            }), 1000);
          } catch (e) {
            console.log('ChoiceJS error fetching endpoint', e);
          }
        });
      }
      //Return
      return;
    })();
  }

  /**
   * Add all of the bindings for the choice
   *
   * @param {HTMLElement} choice - The select we're building the bindings for
   * @return {Promise}
  */
  bindings(choice) {
    var _this13 = this;
    return _asyncToGenerator(function* () {
      //Return
      return yield Promise.all([_this13.bindActiveListener(choice)]);
    })();
  }
}

/**
 * Export the choice function to use
 *
 * @param {NodeElement} target - The target we're adding the choice to
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of choice
 */
export var Choice = /*#__PURE__*/function () {
  var _ref7 = _asyncToGenerator(function* (target, options) {
    //Start a new Choice
    var newBuild = new Builder(target, options);
    //Run the build
    return yield newBuild.create();
  });
  return function Choice(_x5, _x6) {
    return _ref7.apply(this, arguments);
  };
}();

/**
 * Remove the choice by its target
 *
 * @param {NodeElement} target - The target we're removing the choice from
 */
export var DestroyChoice = /*#__PURE__*/function () {
  var _ref8 = _asyncToGenerator(function* (target) {
    //If the target doesnt have the data-choicejs-id attribute, return
    if (!target.hasAttribute('data-choicejs-id')) return false;
    //Remove the choice
    target.parentNode.querySelector('.choicejs').remove();
    //Clear and replace the select
    var new_element = target.cloneNode(true);
    //Remove extra stuff added
    new_element.removeAttribute('data-choicejs-id');
    //Replace it
    target.parentNode.replaceChild(new_element, target);
    //Resolve
    return true;
  });
  return function DestroyChoice(_x7) {
    return _ref8.apply(this, arguments);
  };
}();