function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/*
 * Choice JS
*/

class Builder {
  //Constructor
  constructor(target, userOptions) {
    this.target = target;
    //Build our default options
    this.options = _objectSpread(_objectSpread({}, this.buildDefaultOptions(userOptions)), userOptions);
  }

  /**
   * Create a new choice dropdown
   *
   * @return {Promise}
  */
  create() {
    var _this = this;
    return _asyncToGenerator(function* () {
      //Build the select box
      _this.choice = yield _this.build();
    })();
  }

  /**
   * Build the choice
   *
   * @return {Promise} HTMLElement The new choice
  */
  build() {
    var _this2 = this;
    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var wrapper = document.createElement('div'),
        select = document.createElement('div'),
        choiceId = 'choice-' + Math.random().toString(36).substr(2, 4);

      //Set attributes
      wrapper.classList.add('choicejs');
      wrapper.setAttribute('id', choiceId);
      select.classList.add('choicejs-select');

      //Set the id on the target so we can relate
      _this2.target.setAttribute('data-choicejs-id', "#".concat(wrapper.id));

      //Copy the select boxes attributes
      if (_this2.options.copyAttributes === true) {
        //Only copy over limited attributes
        wrapper.setAttribute('name', _this2.target.getAttribute('name'));
      }

      //Build the whole select and return it
      wrapper.appendChild(select);

      //Configure the options for it
      yield _this2.configureOptions(wrapper);

      //Return
      return wrapper;
    })();
  }
}

/**
 * Export the choice function to use
 *
 * @param {NodeElement} target - The target we're adding the choice to
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of choice
 */
export var Choice = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* (target, options) {
    //Start a new Choice
    var newBuild = new Builder(target, options);
    //Run the build
    return yield newBuild.create();
  });
  return function Choice(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Remove the choice by its target
 *
 * @param {NodeElement} target - The target we're removing the choice from
 */
export var DestroyChoice = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(function* (target) {});
  return function DestroyChoice(_x3) {
    return _ref2.apply(this, arguments);
  };
}();